var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
import React, { Fragment, useCallback, useMemo, useState } from 'react';
import { RefreshScope, useFetch, useRefresh } from 'react-admin-base';
import { FormattedMessage, useIntl } from 'react-intl';
import Select, { components } from "react-select";
import CreatableSelect from 'react-select/creatable';
import { closestCenter, DndContext } from '@dnd-kit/core';
import { restrictToParentElement } from '@dnd-kit/modifiers';
import { arrayMove, horizontalListSortingStrategy, SortableContext, useSortable, } from '@dnd-kit/sortable';
import { CSS } from '@dnd-kit/utilities';
function Option(props) {
    return _jsx(components.Option, Object.assign({}, props, { children: (props.selectProps.children && props.selectProps.children(props.data)) || (props.data.__isNew__ ? _jsx(FormattedMessage, { id: "CREATE_VALUE", values: { text: props.children } }) : props.children) }));
}
function SingleValue(props) {
    return _jsx(components.SingleValue, Object.assign({}, props, { children: (props.selectProps.children && props.selectProps.children(props.data)) || (props.data.__isNew__ ? _jsx(FormattedMessage, { id: "CREATE_VALUE", values: { text: props.children } }) : props.children) }));
}
function MultiValue(props) {
    return _jsx(components.MultiValue, Object.assign({}, props, { children: (props.selectProps.children && props.selectProps.children(props.data)) || props.children }));
}
function EditOrAddIndicator(props) {
    const { className, cx, getStyles, innerProps, isMulti } = props;
    return (_jsx("div", Object.assign({}, innerProps, { className: cx({
            indicator: true,
            'clear-indicator': true
        }, className), css: getStyles('clearIndicator', props), onMouseDown: e => {
            e.stopPropagation();
            e.preventDefault();
            props.selectProps.onAddOrEdit();
        }, children: _jsx("i", { className: "fas " + (props.hasValue && !isMulti ? 'fa-pencil-alt' : 'fa-plus') }) })));
}
function IndicatorsContainer(props) {
    return _jsxs(components.IndicatorsContainer, Object.assign({}, props, { children: [props.selectProps.onAddOrEdit && _jsxs(React.Fragment, { children: [_jsx(EditOrAddIndicator, Object.assign({}, props)), _jsx(components.IndicatorSeparator, Object.assign({}, props))] }), props.children] }));
}
function MultiValueRemove(props) {
    return _jsxs(Fragment, { children: [props.selectProps.onAddOrEdit && _jsx(components.MultiValueRemove, Object.assign({}, props, { innerProps: Object.assign(Object.assign({}, props.innerProps), { onClick: () => props.selectProps.onAddOrEdit(props.data) }), children: _jsx("i", { className: "fas fa-pencil", style: { fontSize: '.75em' } }) })), _jsx(components.MultiValueRemove, Object.assign({}, props))] });
}
const Components = { Option, SingleValue, MultiValue, IndicatorsContainer, MultiValueRemove };
const SortableMultiValue = (props) => {
    const onMouseDown = (e) => {
        e.preventDefault();
        e.stopPropagation();
    };
    const innerProps = Object.assign(Object.assign({}, props.innerProps), { onMouseDown });
    const { attributes, listeners, setNodeRef, transform, transition } = useSortable({
        id: props.selectProps.getOptionValue(props.data),
    });
    const style = {
        transform: CSS.Transform.toString(transform),
        transition,
    };
    return (_jsx("div", Object.assign({ style: style, ref: setNodeRef }, attributes, listeners, { children: _jsx(MultiValue, Object.assign({}, props, { innerProps: innerProps })) })));
};
const SortableMultiValueRemove = (props) => {
    return (_jsx(MultiValueRemove, Object.assign({}, props, { innerProps: Object.assign({ onPointerDown: (e) => e.stopPropagation() }, props.innerProps) })));
};
const SortableComponents = { Option, SingleValue, MultiValue: SortableMultiValue, IndicatorsContainer, MultiValueRemove: SortableMultiValueRemove };
export default function ApiSelect(props) {
    const { disabled, url, getOptionLabel, sortable, getOptionValue, idKey, nameKey, filter, group, onCreateOption, getNewOptionData, isMulti, onChange, value, placeholder, staticOptions } = props;
    const intl = useIntl();
    const [search, setSearch] = useState('');
    const params = useMemo(() => ({ query: search }), [search]);
    const [isMenuOpen, setIsMenuOpen] = useState(false);
    const [hasMenuOpen, setHasMenuOpen] = useState(false);
    const [data, loading, error, update] = useFetch((isMenuOpen && url) || false, params);
    const [creating, setCreating] = useState(false);
    let options = staticOptions || (data && data.data) || data;
    if (group) {
        options = group(options || []);
    }
    const handleCreateOption = useCallback(function (input) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!onCreateOption)
                return null;
            setCreating(true);
            try {
                let option = yield onCreateOption(input);
                if (isMulti) {
                    onChange((value || []).concat([option]));
                }
                else {
                    onChange(option);
                }
            }
            finally {
                setCreating(false);
            }
        });
    }, [onCreateOption, setCreating, isMulti, value]);
    const onMenuOpen = useCallback(function () {
        setIsMenuOpen(true);
        setHasMenuOpen(true);
    }, [setIsMenuOpen, setHasMenuOpen]);
    const onMenuClose = useCallback(function () {
        setIsMenuOpen(false);
    }, [setIsMenuOpen]);
    const _getOptionValue = getOptionValue || ((row) => row[idKey || 'id']);
    const onDragEnd = useCallback(({ active, over }) => {
        if (!active || !over)
            return;
        const oldIndex = value.findIndex((item) => _getOptionValue(item) === active.id);
        const newIndex = value.findIndex((item) => _getOptionValue(item) === over.id);
        const newValue = arrayMove(value, oldIndex, newIndex);
        onChange(newValue);
    }, [_getOptionValue, value, onChange]);
    const Component = (onCreateOption ? CreatableSelect : Select);
    const useSort = isMulti && sortable;
    const elem = _jsx(Component, Object.assign({}, props, { className: 'react-select-container', classNamePrefix: "react-select", onCreateOption: (onCreateOption && handleCreateOption), getNewOptionData: (onCreateOption && (getNewOptionData || ((inputValue) => ({ [nameKey || 'name']: inputValue, __isNew__: true })))) || undefined, inputValue: search, onInputChange: a => setSearch(a), components: (useSort ? SortableComponents : Components), isLoading: !!loading || creating, getOptionLabel: getOptionLabel || ((row) => row[nameKey || 'name']), getOptionValue: _getOptionValue, isDisabled: !!disabled || creating, isClearable: true, isSearchable: true, placeholder: placeholder || intl.formatMessage({ id: 'SELECT' }), options: !options ? [] : ((filter && options.filter(filter)) || options), onMenuOpen: onMenuOpen, onMenuClose: onMenuClose }));
    return _jsx(RefreshScope, { update: update, children: useSort ? _jsx(DndContext, { modifiers: [restrictToParentElement], onDragEnd: onDragEnd, collisionDetection: closestCenter, children: _jsx(SortableContext, { items: (value || []).map(_getOptionValue), strategy: horizontalListSortingStrategy, children: elem }) }) : elem });
}
export function CreateSelect(props) {
    const { Component, onChange, value, isMulti, idKey } = props;
    const update = useRefresh();
    const [isOpen, setIsOpen] = useState(false);
    const isOpenId = isOpen === true ? null : isOpen && isOpen[idKey || "id"];
    const onReload = useCallback(function (data) {
        setIsOpen(false);
        data && onChange(isMulti ? isOpenId ? value.map(b => b.id === isOpenId ? data : b) : (value || []).concat([data]) : data);
        update && update();
    }, [value, onChange, isMulti, isOpenId, isOpen, update, setIsOpen]);
    return _jsxs(Fragment, { children: [isOpen && _jsx(Component, { id: isOpenId || (!isMulti && value && value[idKey || "id"]), onReload: onReload }), _jsx(ApiSelect, Object.assign({}, props, { onAddOrEdit: item => setIsOpen(item || true) }))] });
}
